/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.juddi.monitor.jdbc;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.Timestamp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.juddi.IRegistry;
import org.apache.juddi.monitor.MonitorData;
import org.apache.juddi.util.Config;
import org.apache.juddi.util.jdbc.ConnectionManager;
import org.apache.juddi.util.jdbc.Transaction;
import org.apache.juddi.uuidgen.UUIDGen;
import org.apache.juddi.uuidgen.UUIDGenFactory;

/**
 * @author Anou Mana (anou_mana@users.sourceforge.net)
 */
class MonitorTable
{
  // private reference to the jUDDI logger
  private static Log log = LogFactory.getLog(MonitorTable.class);

  static String insertSQL = null;
  static String selectSQL = null;
  static String deleteSQL = null;

  static
  {
    // buffer used to build SQL statements
    StringBuffer sql = null;

    // build insertSQL
    sql = new StringBuffer(150);
    sql.append("INSERT INTO MONITOR (");
    sql.append("REMOTE_HOST,");
    sql.append("REQUEST_URI,");
    sql.append("CALLED_FUNCTION,");
    sql.append("AUTH_TOKEN,");
    sql.append("UDDI_VERSION,");
    sql.append("LOG_TIME,");
    sql.append("FAULT) ");
    sql.append("VALUES (?,?,?,?,?,?,?)");
    insertSQL = sql.toString();

    // build selectSQL
    sql = new StringBuffer(200);
    sql.append("SELECT ");
    sql.append("LOG_TIME,");
    sql.append("REMOTE_HOST,");
    sql.append("REQUEST_URI,");
    sql.append("CALLED_FUNCTION,");
    sql.append("AUTH_TOKEN,");
    sql.append("UDDI_VERSION,");
    sql.append("FAULT) ");
    sql.append("FROM MONITOR ");
    sql.append("WHERE FAULT=? ");
    selectSQL = sql.toString();

    // build deleteSQL
    sql = new StringBuffer(100);
    sql.append("DELETE FROM MONITOR ");
    sql.append("WHERE FAULT=? ");
    deleteSQL = sql.toString();
  }

  /**
   * Insert new row into the MONITOR table.
   *
   * @param Monitor
   * @param connection JDBC connection
   * @throws java.sql.SQLException
   */
  public static void insert(MonitorData data,Connection connection)
    throws java.sql.SQLException
  {
    if (data == null)
      return; // everything is valid but no elements to insert

    PreparedStatement statement = null;
    Timestamp timeStamp = new Timestamp(System.currentTimeMillis());

    try
    {
      statement = connection.prepareStatement(insertSQL);
      statement.setString(1,data.getRemoteHost());
      statement.setString(2,data.getRequestURI());
      statement.setString(3,data.getCalledFunction());
      statement.setString(4,data.getAuthToken());
      statement.setString(5,data.getVersion());
      statement.setTimestamp(6,timeStamp);
      statement.setString(7,data.getFault());

        log.debug("insert into MONITOR table:\n\n\t" + insertSQL +
          "\n\t REMOTE_HOST=" + data.getRemoteHost() +
          "\n\t REQUEST_URI=" + data.getRequestURI() +
          "\n\t CALLED_FUNCTION=" + data.getCalledFunction() +
          "\n\t AUTH_TOKEN=" + data.getAuthToken()+
          "\n\t UDDI_VERSION=" + data.getVersion() +
          "\n\t LOG_TIME=" + timeStamp.getTime() +
          "\n\t FAULT=" + data.getFault() + "\n");

        statement.executeUpdate();
    }
    finally
    {
      try {
        statement.close();
      }
      catch (Exception e) { /* ignored */ }
    }
  }

  /***************************************************************************/
  /***************************** TEST DRIVER *********************************/
  /***************************************************************************/


  public static void main(String[] args)
    throws Exception
  {
    // make sure we're using a DBCP DataSource and
    // not trying to use JNDI to aquire one.
    Config.setStringProperty("juddi.useConnectionPool","true");

    Connection conn = null;
    try {
      conn = ConnectionManager.aquireConnection();
      test(conn);
    }
    finally {
      if (conn != null)
        conn.close();
    }
  }

  public static void test(Connection connection)
    throws Exception
  {
    UUIDGen uuidgen = UUIDGenFactory.getUUIDGen();
    Transaction txn = new Transaction();

    if (connection != null)
    {
      try
      {
        MonitorData data = new MonitorData("123.234.345.456", "testFunction");

        data.setAuthToken("");
        data.setUddiVersion(IRegistry.UDDI_V2_GENERIC);
        data.setFault("None");

        // begin a new transaction
        txn.begin(connection);

        // insert a new monitor log
        MonitorTable.insert(data,connection);

        // commit the transaction
        txn.commit();
      }
      catch(Exception ex)
      {
        try { txn.rollback(); }
        catch(java.sql.SQLException sqlex) { sqlex.printStackTrace(); }
        throw ex;
      }
    }
  }
}
